Player = {}

function Player:new()
	object = {
	x = 500,
	y = 500,
	x_vel = 0,
	y_vel = 0,
	x_accel = 700,
	y_accel = 50,
	friction = 305,
	gravity = 600,
	collision_x_offset = 2,
	collision_y_offset = 2,
	is_moving = false, -- TRUE if player changed position last frame
	previous_x = 0,

	run_speed = {65, 200, 275},
	run_accel = {2000, 860, 455},
	run_times = {0.08, 1.8},
	run_timer = 0,
	gear = 1,

	debug = "",

	width  = 24,
	height = 36,

	on_floor = true,
	on_slope = false,

	bonus_friction = 0,
	bonus_speed = 0
	}

	setmetatable( object, {__index = Player} )
	return object
end

function Player:update(dt)
	mx, my = love.mouse.getPosition()
	mouse()

	if love.keyboard.isDown("s") then
		--self.on_slope = true
		self.bonus_friction = -200
		self.x = 500
		self.y = 500
		self.y_vel = 0
	else
		--self.on_slope = false
		self.bonus_friction = 0
	end

	-- apply friction
	if self.x_vel > 0 then
		self.x_vel = self.x_vel - (self.friction * dt) - (self.bonus_friction * dt)
		if self.x_vel < 0 then self.x_vel = 0 end
	end
	if self.x_vel < 0 then
		self.x_vel = self.x_vel + (self.friction * dt) + (self.bonus_friction * dt)
		if self.x_vel > 0 then self.x_vel = 0 end
	end

	if love.keyboard.isDown( i_left  ) then self:moveLeft(dt) end
	if love.keyboard.isDown( i_right ) then self:moveRight(dt) end
	if love.keyboard.isDown( i_up    ) then  end
	if love.keyboard.isDown( i_down  ) then  end

	self.previous_x = self.x

	-- collision detection here

	--local tile
	self.y_vel = self.y_vel + (self.gravity * dt)

	local next_x = self.x + (self.x_vel * dt)

	-- X-AXIS --

	if self.x_vel > 0 then
		if	collision(1, next_x + (self.width/2) - self.collision_x_offset, self.y + (self.height / 2) - self.collision_y_offset) or
			collision(1, next_x + (self.width/2) - self.collision_x_offset, self.y - (self.height / 2) + 1 + self.collision_y_offset) then
			self.x_vel = 0
			self.x = (math.ceil((next_x + (self.width/2) - self.collision_x_offset) / tile_size) - 1) * tile_size - (self.width / 2) + self.collision_x_offset
		else
			self.x = next_x
		end
	end

	if self.x_vel < 0 then
		--if	collision(1, next_x - (self.width/2) + self.collision_x_offset, self.y + (self.height / 2) - self.collision_y_offset) or
		--	collision(1, next_x - (self.width/2) + self.collision_x_offset, self.y - (self.height / 2) + 1 + self.collision_y_offset) then
		--	self.x_vel = 0
		--	self.x = (math.ceil(next_x / tile_size) - 1) * tile_size + (self.width / 2) - self.collision_x_offset
		--end
		if	collision(1, next_x - (self.width/2) + self.collision_x_offset, self.y + (self.height / 2) - self.collision_y_offset) or
			collision(1, next_x - (self.width/2) + self.collision_x_offset, self.y - (self.height / 2) + 1 + self.collision_y_offset) then
			-- if player collide with wall check if there is slope behind it
			if 	collision(2, next_x - (self.width/2) + self.collision_x_offset + tile_size, self.y + (self.height / 2) - self.collision_y_offset) or
				collision(2, next_x - (self.width/2) + self.collision_x_offset + (2 * tile_size), self.y + (self.height / 2) - self.collision_y_offset) then
				
			self.x = next_x
			else
				self.x_vel = 0
				self.x = math.ceil((next_x - (self.width/2) + self.collision_x_offset) / tile_size) * tile_size + (self.width / 2) - self.collision_x_offset
			end
		else
			self.x = next_x
		end
	end

	next_y = self.y + (self.y_vel * dt)

	-- Y-AXIS --

	if self.y_vel > 0 then
		-- regular blocks
		if	collision(1, self.x + (self.width/2) - self.collision_x_offset, next_y + (self.height / 2)) or
			collision(1, self.x - (self.width/2) + self.collision_x_offset, next_y + (self.height / 2)) then
			self.y_vel = 0
			self.y = (math.ceil((next_y + (self.height / 2)) / tile_size) - 1) * tile_size - (self.height / 2)
		else
			self.y = next_y
		end
	end

	-- SLOPES --

	-- jeśli nie ma kolizji ze slopem, sprawdź czy jest slope tile niżej TYLKO jeśli on_slope jest TRUE!

	p1 = {math.round(self.x - (self.width/2) + 3), math.round(next_y + (self.height / 2))}

	if	collision(2, p1[1], p1[2]) 	then
			tile = get_tile(p1[1], p1[2])
	end

	-- if player is in wall and have slope behind it
	if collision(0, p1[1], p1[2]) and collision(2, p1[1] + (tile_size / 2), p1[2]) then
		tile = get_tile(p1[1], p1[2])

		local t = (math.floor(p1[1]) - (tile["x"] * tile_size)) / tile_size
		local floor_y = ((1-t) * (24)) + (t * 1);

		self.y = ((tile["y"] + 1) * tile_size) - (self.height / 2) - math.ceil(floor_y) - tile_size
		self.y_vel = 0
		self.on_slope = true
	end

	-- if player is in air and have slope undeneath it
	if collision(0, p1[1], p1[2]) and collision(2, p1[1], p1[2] + (tile_size / 2)) then
		self.y = self.y + tile_size
		--self.on_slope = true
	end

	--[[

	Są trzy rodzaje kolidowania: zwykły, wjazd w ścianę, puste pole
	Jeśli wjeżdżamy w ścianę, koniecznie przyklejamy do górnego pola
	Jeśli nic nie ma, przyklejamy do dolnego pola

	Po wjeździe w ścianę wiadomo, że nie jesteśmy wyżej niż powinniśmy, także skakanie będzie działało poprawnie
	Na pustym polu przyklejanie powinno działać tylko wtedy, jeśli wiemy, że gracz jest na rampie
	Jeśli nie jesteśmy na rampie sprawdzamy czy przyklejenie (z odpowiednią wysokością) jest równe lub większe
	faktycznej pozycji Y
	Jeśli nie, pozwalamy graczowi opadać. Jeśli tak, przyciągamy do odpowiedniego poziomu, on_slope = true i
	następny raz będzie wiadomo, że przyciągamy do odpowiedniego poziomu.

	]]

	if	collision(2, p1[1], p1[2]) then

			local t = (math.floor(p1[1]) - (tile["x"] * tile_size)) / tile_size
			local floor_y = ((1-t) * (24)) + (t * 1);

			debug2 = ((tile["y"] + 1) * tile_size) - (self.height / 2) - math.ceil(floor_y)
			
			self.y_vel = 0
			debug = "Kolizja"

			--self.y = (tile["y"] * tile_size) - (self.height / 2) - 12
			--if math.floor(self.y) >=  math.floor(((tile["y"] + 1) * tile_size) - (self.height / 2) - math.ceil(floor_y)) then
				self.y = ((tile["y"] + 1) * tile_size) - (self.height / 2) - math.ceil(floor_y)
			--end
			-- dociśnij gracza jeśli jest niżej niż slope
			--if self.y_vel == 0 then self.on_slope = true end
	else
		-- if there is no slope in the point but there is one underneath and you are on slope 
		--if collision(2, p1[1] + (tile_size / 2), p1[2]) then
		--	self.y_vel = 0
		--	self. on_slope = true
		--else
			debug = ""
			--self.on_slope = false
		--end
	end

	if self.previous_x ~= self.x then self.is_moving = true else self.is_moving = false end
	if self.is_moving and (love.keyboard.isDown( i_left ) or love.keyboard.isDown( i_right )) then
		self.run_timer = self.run_timer + dt
	end
	if self.run_timer >= 0   then self.gear = 1 end
	if self.run_timer > self.run_times[1] then self.gear = 2 end
	if self.run_timer > self.run_times[2] then self.gear = 3 end
end

-- Slide on slopes will be same thing with bigger accell and max speed
function Player:moveRight(dt)
	if self.x_vel <= self.run_speed[self.gear] then
		self.x_vel = self.x_vel + (self.run_accel[self.gear] * dt)
		if self.x_vel > self.run_speed[self.gear] then self.x_vel = self.run_speed[self.gear] end
	end
end

function Player:moveLeft(dt)
	if self.x_vel >= -self.run_speed[self.gear] then
		self.x_vel = self.x_vel - (self.run_accel[self.gear] * dt)
		if self.x_vel < -self.run_speed[self.gear] then self.x_vel = -self.run_speed[self.gear] end
	end
end

function Player:draw()
	love.graphics.setColor( 180, 23, 242 )
	love.graphics.rectangle("fill", self.x - (self.width/2) + self.collision_x_offset + 1, math.round(next_y + (self.height / 2)), 2, 1)
	love.graphics.rectangle("fill", self.x - (self.width/2) + self.collision_x_offset + 1, math.round(next_y + (self.height / 2)), 2, 1)
	love.graphics.setColor( 242, 242, 242 )

	love.graphics.print(debug, 20, 500)
	love.graphics.print(tostring(debug2), 20, 600)
	love.graphics.print(p1[1], 20, 515)
	love.graphics.print(p1[2], 20, 530)
	if tile ~= nil then love.graphics.print(tile["x"].." "..tile["y"], 20, 545) end

	if self.on_slope then love.graphics.setColor( 133, 13, 133 )
	else love.graphics.setColor( 242, 242, 242 )
	end
	
	love.graphics.rectangle("fill", math.round(self.x - (self.width/2)) + 2, math.round(self.y - (self.height/2)), self.width - 2, self.height)
	love.graphics.setColor( 0, 0, 0 )
	love.graphics.rectangle("fill", self.x, self.y, 1, 1)
	-- collision points Right side
	love.graphics.rectangle("fill", self.x + (self.width/2) - self.collision_x_offset, self.y + (self.height/2) - 1 - self.collision_y_offset, 1, 1)
	love.graphics.rectangle("fill", self.x + (self.width/2) - self.collision_x_offset, self.y - (self.height/2) + self.collision_y_offset, 1, 1)
	-- collision points Left side
	love.graphics.rectangle("fill", self.x - (self.width/2) + self.collision_x_offset - 1, self.y + (self.height/2) - 1 - self.collision_y_offset, 1, 1)
	love.graphics.rectangle("fill", self.x - (self.width/2) + self.collision_x_offset - 1, self.y - (self.height/2) + self.collision_y_offset, 1, 1)
	
	-- collision points Down side
	love.graphics.setColor( 255, 255, 255 )
	love.graphics.rectangle("fill", self.x - (self.width/2) + self.collision_x_offset, self.y + (self.height / 2), 1, 1)
	love.graphics.rectangle("fill", self.x + (self.width/2) - 1 - self.collision_x_offset, self.y + (self.height / 2), 1, 1)

	love.graphics.setColor( 255, 255, 255 )

	love.graphics.polygon('fill', 100, 100, 124, 100, 100, 76)

	love.graphics.print("X: "..self.x, 200, 200)
	love.graphics.print("Y: "..self.y, 200, 215)
	love.graphics.print("X: "..self.x_vel, 110, 200)
	love.graphics.print("Y: "..self.y_vel, 110, 215)
	love.graphics.print("Run timer: "..self.run_timer, 200, 230)
	love.graphics.print("Gear: "..self.gear, 200, 260)
	love.graphics.print("Acceleration: "..tostring(self.run_accel[self.gear]), 200, 275)
	love.graphics.print("Velocity: "..math.round(self.x_vel), 200, 290)
	--love.graphics.print("Mouse_X: "..mx, 200, 305)
	--love.graphics.print("Mouse_Y: "..my, 200, 330)
	if self.is_moving then love.graphics.print("R U N I N G", 350, 200) end
	if self.on_slope then love.graphics.print("On Slope", 350, 215) end

	-- // --

	local x,y = 500, 200
	love.graphics.print("Run speed",x,y)
	love.graphics.print(self.run_speed[1],x,y+15)
	love.graphics.print(self.run_speed[2],x,y+30)
	love.graphics.print(self.run_speed[3],x,y+45)

	love.graphics.print("Run accel",x+140,y)
	love.graphics.print(self.run_accel[1],x+140,y+15)
	love.graphics.print(self.run_accel[2],x+140,y+30)
	love.graphics.print(self.run_accel[3],x+140,y+45)

	love.graphics.print("Run times",x+280,y)
	love.graphics.print(self.run_times[1],x+280,y+15)
	love.graphics.print(self.run_times[2],x+280,y+30)

	love.graphics.print("Friction",x+420,y)
	love.graphics.print(self.friction,x+420,y+15)
end

function mouse()
	local x, y = 474, 217
	if love.mouse.isDown( "l" ) then
		if is_inside(x,    y) then player.run_speed[1] = player.run_speed[1] - (125 * min_dt) end
		if is_inside(x+67, y) then player.run_speed[1] = player.run_speed[1] + (125 * min_dt) end
		if is_inside(x,    y+15) then player.run_speed[2] = player.run_speed[2] - (125 * min_dt) end
		if is_inside(x+67, y+15) then player.run_speed[2] = player.run_speed[2] + (125 * min_dt) end
		if is_inside(x,    y+30) then player.run_speed[3] = player.run_speed[3] - (125 * min_dt) end
		if is_inside(x+67, y+30) then player.run_speed[3] = player.run_speed[3] + (125 * min_dt) end

		if is_inside(x+140,    y)    then player.run_accel[1] = player.run_accel[1] - (125 * min_dt) end
		if is_inside(x+140+67, y)    then player.run_accel[1] = player.run_accel[1] + (125 * min_dt) end
		if is_inside(x+140,    y+15) then player.run_accel[2] = player.run_accel[2] - (125 * min_dt) end
		if is_inside(x+140+67, y+15) then player.run_accel[2] = player.run_accel[2] + (125 * min_dt) end
		if is_inside(x+140,    y+30) then player.run_accel[3] = player.run_accel[3] - (125 * min_dt) end
		if is_inside(x+140+67, y+30) then player.run_accel[3] = player.run_accel[3] + (125 * min_dt) end

		if is_inside(x+140+140,    y)    then player.run_times[1] = player.run_times[1] - (125 * min_dt / 200) end
		if is_inside(x+140+140+67, y)    then player.run_times[1] = player.run_times[1] + (125 * min_dt / 200) end
		if is_inside(x+140+140,    y+15) then player.run_times[2] = player.run_times[2] - (125 * min_dt / 200) end
		if is_inside(x+140+140+67, y+15) then player.run_times[2] = player.run_times[2] + (125 * min_dt / 200) end

		if is_inside(x+140+140+140,    y)    then player.friction = player.friction - (125 * min_dt) end
		if is_inside(x+140+140+140+67, y)    then player.friction = player.friction + (125 * min_dt) end
	end
end

function is_inside(x, y)
	if mx >= x and mx <= x + 25 and my >= y and my <= y+10 then
		return true
	else
		return false
	end
end