
	file = love.filesystem.newFile('debug.txt')

Player = {}

function Player:new()
	object = {
	x = 385,
	y = 560,
	x_vel = 0,
	y_vel = 0,
	max_y_vel = 0,
	x_accel = 700,
	y_accel = 50,
	friction = 305,
	gravity = 1650,
	collision_x_offset = 2,
	collision_y_offset = 2,
	is_moving = false, -- TRUE if player changed position last frame
	previous_x = 0,

	run_speed = {65, 200, 275},
	run_accel = {2000, 860, 455},
	run_times = {0.08, 1.8},
	run_timer = 0,
	gear = 1,

	jump_state = "none",
	default_jump_power = 295,
	jump_power = 295,
	accel_timer = 0.15,
	slope_timer = 0,
	has_jumped = false,
	doublejump = true,

	debug = "",

	width  = 24,
	height = 36,

	on_floor = true,
	on_slope = false,
	hit_slope = false,
	is_sliding = 0,
	is_skiing = false,
	current_slope = {}, -- {[1/2/3], [l/r]} _ / |

	bonus_friction = 0,
	bonus_speed = 0,
	drop_speed = 0
	}

	setmetatable( object, {__index = Player} )
	return object
end

function Player:update(dt)
	mx, my = love.mouse.getPosition()
	mouse()

	-- timers
	if self.slope_timer > 0 then self.slope_timer = self.slope_timer - dt end

	--if love.keyboard.isDown("s") then
	--	--self.on_slope = true
	--	self.bonus_friction = -200
	--	self.x = 500
	--	self.y = 500
	--	self.y_vel = 0
	--else
	--	--self.on_slope = false
	--	self.bonus_friction = 0
	--end

	-- apply friction
	--[[

		Going UP the slope should increase the friction.
		Going DOWN the slope should decrease the friction.
		Standing on slope should drag you in one direction.

	]]
	if self.on_slope then
		if	   self.current_slope[1] == 1 then
			self.bonus_friction = 50
			self.bonus_speed = 0
		elseif self.current_slope[1] == 2 then
			self.bonus_friction = 150
			self.bonus_speed = 0
		elseif self.current_slope[1] == 3 then
			self.bonus_friction = 300
			if self.current_slope[2] == 'l' then
				self.bonus_speed = 200
			else
				self.bonus_speed = -200
			end
		end

		if self.current_slope[2] == 'r' then
			if self.x_vel > 0 then
				self.bonus_friction = - self.bonus_friction
			end
		elseif self.current_slope[2] == 'l' then
			if self.x_vel < 0 then
				self.bonus_friction = - self.bonus_friction
			end
		end
	else
		self.bonus_friction = 0
		self.bonus_speed = 0
	end

	--[[

		You can ski on slopes which means there is
		less friction and you get more speed from
		droping on slopes.
		Also it pushes you off certain slopes.

	]]

	if self.is_skiing and self.on_slope then
		if self.current_slope[1] == 2 then
			if self.current_slope[2] == 'l' then
				self.bonus_speed = self.bonus_speed + 100
			else
				self.bonus_speed = self.bonus_speed - 100
			end
		end
		if self.current_slope[1] == 3 then
			if self.current_slope[2] == 'l' then
				self.bonus_speed = self.bonus_speed + 50
			else
				self.bonus_speed = self.bonus_speed - 50
			end
		end
		if (self.current_slope[2] == 'l' and self.x_vel > 0) or (self.current_slope[2] == 'r' and self.x_vel < 0) then
			self.bonus_friction = self.bonus_friction + 200
		end
	end

	if self.x_vel > 0 then
		self.x_vel = self.x_vel - (self.friction * dt) + (self.bonus_friction * dt)
		if self.x_vel < 0 then self.x_vel = 0 end
	end
	if self.x_vel < 0 then
		self.x_vel = self.x_vel + (self.friction * dt) - (self.bonus_friction * dt)
		if self.x_vel > 0 then self.x_vel = 0 end
	end
	if self.bonus_speed ~= 0 then
		self.x_vel = self.x_vel + (self.bonus_speed * dt)
	end

	if math.abs(self.x_vel) < 50 or not self.on_slope then self.is_sliding = 0 end
	if love.keyboard.isDown( i_left  ) then self:moveLeft(dt) end
	if love.keyboard.isDown( i_right ) then self:moveRight(dt) end
	if love.keyboard.isDown( i_up    ) then  end
	if love.keyboard.isDown( i_down  ) then self.is_skiing = true else self.is_skiing = false end
	if love.mouse.isDown   ( i_jump  ) then
		-- if the jump has no state check if we can jump
		if self.jump_state == "none" then
			-- when you can jump:
			if self.on_floor or self.doublejump then
				-- use double jump if you double jump
				if not self.on_floor and self.doublejump then
					self.doublejump = false
				end
				-- jump breaks sliding
				self.is_sliding = 0
				-- count jump power
				if self.x_vel < 0 then
					-- find out if player jumps from the slope
					local p1 = {math.round(self.x - (self.width/2) + 3), math.round(next_y + (self.height / 2))}
					local p2 = {math.round(self.x - (self.width/2) + 3) + 5, math.round(next_y + (self.height / 2) + 5)}
					local tile
					if collision(2, p1[1], p1[2]) and self.on_slope then
						tile = deepcopy(get_tile(p1[1], p1[2]))
					else
						if collision(2, p2[1], p2[2]) and self.on_slope then
							tile = deepcopy(get_tile(p2[1], p2[2]))
						else
							tile = nil
						end
					end
					if tile ~= nil then
						-- Left 45 slope
						if tile['l_edge'] == 24 and tile['r_edge'] == 1 then
							ttt = 1 + (-self.x_vel - 230) / 230
							local bonus_y_vel = ttt * 100
							local bonus_x_vel = ttt * 100
							if bonus_y_vel > 0 then self.jump_power = self.default_jump_power + bonus_y_vel end
							self.x_vel = self.x_vel + bonus_x_vel
						
						-- Left 23 slope
						elseif (tile['l_edge'] == 12 and tile['r_edge'] == 1) or (tile['l_edge'] == 24 and tile['r_edge'] == 12) then
							ttt = (1 + (-self.x_vel - 230) / 230) * 0.50
							local bonus_y_vel = ttt * 100
							local bonus_x_vel = ttt * 100
							if bonus_y_vel > 0 then self.jump_power = self.default_jump_power + bonus_y_vel end
							self.x_vel = self.x_vel + bonus_x_vel
						
						-- Left 69 slope
						elseif (tile['l_edge'] == 48 and tile['r_edge'] == 1) or (tile['l_edge'] == 24 and tile['r_edge'] == -24) then
							ttt = (1 + (-self.x_vel - 230) / 230) * 1.50
							local bonus_y_vel = ttt * 100
							local bonus_x_vel = ttt * 100
							if bonus_y_vel > 0 then self.jump_power = self.default_jump_power + bonus_y_vel end
							self.x_vel = self.x_vel + bonus_x_vel
						else
							self.y_vel = - self.default_jump_power
							self.jump_power = self.default_jump_power
						end
					else
						self.y_vel = - self.default_jump_power
						self.jump_power = self.default_jump_power
					end
				end

				if self.x_vel > 0 then
				-- if you are on slope jump count jump_power! Remember to subtrack x_vel too! PROMODE!
					local p1 = {math.round(self.x + (self.width/2) - 3), math.round(next_y + (self.height / 2))}
					local p2 = {math.round(self.x + (self.width/2) - 3) - 5, math.round(next_y + (self.height / 2) - 5)}
					local tile
					if collision(2, p1[1], p1[2]) and self.on_slope then
						tile = deepcopy(get_tile(p1[1], p1[2]))
					else
						if collision(2, p2[1], p2[2]) and self.on_slope then
							tile = deepcopy(get_tile(p2[1], p2[2]))
						else
							tile = nil
						end
					end
					if tile ~= nil then
						-- set here jump power that will be used everywhere
						-- Right 45 slope
						if tile['r_edge'] == 24 and tile['l_edge'] == 1 then
							ttt = 1 + (self.x_vel - 230) / 230
							local bonus_y_vel = ttt * 100
							local bonus_x_vel = ttt * 100
							if bonus_y_vel > 0 then self.jump_power = self.default_jump_power + bonus_y_vel end
							self.x_vel = self.x_vel - bonus_x_vel
						
						-- Right 23 slope
						elseif (tile['r_edge'] == 12 and tile['l_edge'] == 1) or (tile['r_edge'] == 24 and tile['l_edge'] == 12) then
							ttt = (1 + (self.x_vel - 230) / 230) * 0.50
							local bonus_y_vel = ttt * 100
							local bonus_x_vel = ttt * 100
							if bonus_y_vel > 0 then self.jump_power = self.default_jump_power + bonus_y_vel end
							self.x_vel = self.x_vel - bonus_x_vel
						
						-- Right 69 slope
						elseif (tile['r_edge'] == 48 and tile['l_edge'] == 1) or (tile['r_edge'] == 24 and tile['l_edge'] == -24) then
							ttt = (1 + (self.x_vel - 230) / 230) * 1.50
							local bonus_y_vel = ttt * 100
							local bonus_x_vel = ttt * 100
							if bonus_y_vel > 0 then self.jump_power = self.default_jump_power + bonus_y_vel end
							self.x_vel = self.x_vel - bonus_x_vel

							self.slope_timer = 0.8
						else
							self.y_vel = - self.default_jump_power
							self.jump_power = self.default_jump_power
						end
					else
						self.y_vel = - self.default_jump_power
						self.jump_power = self.default_jump_power
					end
				end

				if self.x_vel == 0 then
					self.y_vel = - self.default_jump_power
					self.jump_power = self.default_jump_power
				end

				self.on_floor = false
				self.on_slope = false
				self.has_jumped = true
				-- turn on accelerating
				self.jump_state = "accelerating"
			end
		end

		if self.jump_state == "accelerating" then
			if self.accel_timer > 0 then self.accel_timer = self.accel_timer - dt end
			if self.accel_timer > 0 then
				self.y_vel = -self.jump_power
			end
		end
	else
		-- if you stop pressing jump while accelerating, stop acceleration
		if self.jump_state == "accelerating" then self.jump_state = "none" end
		self.accel_timer = 0.15
	end

	self.previous_x = self.x

	self:collision_check(dt)


	if self.y_vel > self.max_y_vel then
		self.max_y_vel = self.y_vel
	end

--	if slope['d'] == nil then self.on_slope = false end

	--- END ---

	if self.previous_x ~= self.x then self.is_moving = true else self.is_moving = false end
	if self.is_moving and (love.keyboard.isDown( i_left ) or love.keyboard.isDown( i_right )) then
		self.run_timer = self.run_timer + dt
	else
		self.run_timer = 0
	end
	if self.run_timer >= 0   then self.gear = 1 end
	if self.run_timer > self.run_times[1] then self.gear = 2 end
	--if self.run_timer > self.run_times[2] then self.gear = 3 end
	--if self.gear == 3 and math.abs(self.x_vel) < self.run_speed[2] then self.gear = 2; self.run_timer = 0 end
end

-- Slide on slopes will be same thing with bigger accell and max speed
function Player:moveRight(dt)
	if self.x_vel <= self.run_speed[self.gear] and
		self.is_sliding >= 0 then
		self.x_vel = self.x_vel + (self.run_accel[self.gear] * dt)
		if self.x_vel > self.run_speed[self.gear] then self.x_vel = self.run_speed[self.gear] end
	end
end

function Player:moveLeft(dt)
	if self.x_vel >= -self.run_speed[self.gear] and
		self.is_sliding <= 0 then
		self.x_vel = self.x_vel - (self.run_accel[self.gear] * dt)
		if self.x_vel < -self.run_speed[self.gear] then self.x_vel = -self.run_speed[self.gear] end
	end
end

function Player:ski()

end


--function Player:moveUp(dt)
--	self.y = self.y - (50 * dt)
--end
--
--function Player:moveDown(dt)
--	self.y = self.y + (50 * dt)
--end


function Player:draw()
--	love.graphics.setColor( 255, 255, 255, 100 )
--	if slope['x'] ~= nil and slope['y'] ~= nil then
--		love.graphics.rectangle("fill", (slope['x'] - 1) * tile_size, (slope['y'] - 1) * tile_size, tile_size, tile_size)
--	end
--	love.graphics.setColor( 255, 255, 255 )

--	love.graphics.print(tostring(l_tile1['distance']), 450, 220, 0, 2)
--	love.graphics.print(tostring(l_tile2['distance']), 600, 220, 0, 2)
--	love.graphics.print(tostring(l_tile3['distance']), 525, 180, 0, 2)
--	love.graphics.setColor( 44, 22, 255 )
--	love.graphics.print(tostring(l_tile4['distance']), 525, 260, 0, 2)
--	love.graphics.setColor( 255, 255, 255 )
--
--	love.graphics.print(tostring(r_tile1['distance']), 650, 220, 0, 2)
--	love.graphics.print(tostring(r_tile2['distance']), 800, 220, 0, 2)
--	love.graphics.print(tostring(r_tile3['distance']), 725, 180, 0, 2)
--	love.graphics.setColor( 44, 22, 255 )
--	love.graphics.print(tostring(r_tile4['distance']), 725, 260, 0, 2)
--	love.graphics.setColor( 255, 255, 255 )

	love.graphics.setColor( 180, 23, 242 )
	--love.graphics.rectangle("fill", self.x - (self.width/2) + self.collision_x_offset + 1, math.round(next_y + (self.height / 2)), 2, 1)
	--love.graphics.rectangle("fill", self.x - (self.width/2) + self.collision_x_offset + 1, math.round(next_y + (self.height / 2)), 2, 1)
	love.graphics.setColor( 242, 242, 242 )

	--if self.doublejump then
	if ceil_y ~= nil then love.graphics.print(tostring(ceil_y), 20, 700) end
	love.graphics.print(tostring(self.bonus_friction), 40, 680)
	--end
	--love.graphics.print('t' .. tostring(t), 20, 480)
	--if floor_y ~= nil then love.graphics.print('floor_y' .. tostring(floor_y), 20, 500) end
--	if slope['distance'] ~= nil then love.graphics.print('...: '..slope['distance'], 200, 500) end
--	if slope['d'] ~= nil then love.graphics.print('...: '..slope['d'], 200, 520) end
	--if p1 ~= nil then love.graphics.print(p1[1], 20, 515) end
	--if p1 ~= nil then love.graphics.print(p1[2], 20, 530) end
	--if tile ~= nil then love.graphics.print(tile["x"].." "..tile["y"], 20, 545) end

	if self.on_slope then love.graphics.setColor( 133, 13, 133 )
	else love.graphics.setColor( 242, 242, 242 )
	end
	
	love.graphics.rectangle("fill", math.round(self.x - (self.width/2)), math.round(self.y - (self.height/2)), self.width, self.height)
	love.graphics.setColor( 0, 0, 0 )
	love.graphics.rectangle("fill", self.x, self.y, 1, 1)
--	-- collision points Right side
--	love.graphics.rectangle("fill", self.x + (self.width/2) - self.collision_x_offset, self.y + (self.height/2) - 1 - self.collision_y_offset, 1, 1)
--	love.graphics.rectangle("fill", self.x + (self.width/2) - self.collision_x_offset, self.y - (self.height/2) + self.collision_y_offset, 1, 1)
--	-- collision points Left side
--	love.graphics.rectangle("fill", self.x - (self.width/2) + self.collision_x_offset - 1, self.y + (self.height/2) - 1 - self.collision_y_offset, 1, 1)
--	love.graphics.rectangle("fill", self.x - (self.width/2) + self.collision_x_offset - 1, self.y - (self.height/2) + self.collision_y_offset, 1, 1)
--	
--	-- collision points Down side
--	love.graphics.setColor( 255, 255, 255 )
--	love.graphics.rectangle("fill", self.x - (self.width/2) + self.collision_x_offset, self.y + (self.height / 2), 1, 1)
--	love.graphics.rectangle("fill", self.x + (self.width/2) - 1 - self.collision_x_offset, self.y + (self.height / 2), 1, 1)
--
	love.graphics.setColor( 255, 255, 255 )
--
--	love.graphics.polygon('fill', 100, 100, 124, 100, 100, 76)
--
	love.graphics.print("X: "..self.x, 200, 200)
	love.graphics.print("Y: "..self.y, 200, 215)
	love.graphics.print("X: "..self.x_vel, 110, 200)
	love.graphics.print("Y: "..self.y_vel, 110, 215)
	love.graphics.print("Run timer: "..self.run_timer, 200, 230)
	love.graphics.print("Gear: "..self.gear, 200, 260)
	love.graphics.print("Acceleration: "..tostring(self.run_accel[self.gear]), 200, 275)
	love.graphics.print("Velocity: "..math.round(self.x_vel), 200, 290)
	--love.graphics.print("Mouse_X: "..mx, 200, 305)
	--love.graphics.print("Mouse_Y: "..my, 200, 330)
--	if self.is_moving then love.graphics.print("R U N I N G", 350, 200) end
--	if self.on_slope then love.graphics.print("On Slope", 350, 215) end

	-- // --

--	local x,y = 500, 200
--	love.graphics.print("Run speed",x,y)
--	love.graphics.print(self.run_speed[1],x,y+15)
--	love.graphics.print(self.run_speed[2],x,y+30)
--	love.graphics.print(self.run_speed[3],x,y+45)
--
--	love.graphics.print("Run accel",x+140,y)
--	love.graphics.print(self.run_accel[1],x+140,y+15)
--	love.graphics.print(self.run_accel[2],x+140,y+30)
--	love.graphics.print(self.run_accel[3],x+140,y+45)
--
--	love.graphics.print("Run times",x+280,y)
--	love.graphics.print(self.run_times[1],x+280,y+15)
--	love.graphics.print(self.run_times[2],x+280,y+30)
--
--	love.graphics.print("Friction",x+420,y)
--	love.graphics.print(self.friction,x+420,y+15)
--
--
--
--	love.graphics.rectangle("fill", math.round(self.x - (self.width/2) + 3), math.round(next_y + (self.height / 2)), 1, 1)
--	love.graphics.setColor( 242, 23, 18 )
--	love.graphics.rectangle("fill", math.round(self.x - (self.width/2) + 3), math.round(next_y + (self.height / 2)), 12, 1)
--	love.graphics.rectangle("fill", math.round(self.x - (self.width/2) + 3) - 12, math.round(next_y + (self.height / 2)), 12, 1)
--	love.graphics.rectangle("fill", math.round(self.x - (self.width/2) + 3), math.round(next_y + (self.height / 2)), 1, 12)
--	love.graphics.rectangle("fill", math.round(self.x - (self.width/2) + 3), math.round(next_y + (self.height / 2)) - 12, 1, 12)
--	love.graphics.setColor( 42, 242, 242 )
--	if l_tile1['x'] ~= nil then love.graphics.rectangle("fill", l_tile1['x'] * tile_size, l_tile1['y'] * tile_size, 1, 1) end
--	if l_tile4['x'] ~= nil then love.graphics.print(tostring(l_tile4['x']) .. ' ' .. tostring(l_tile4['d']), 21, 280) end
--	love.graphics.rectangle("fill", p1[1], p1[2] + (tile_size / 2), 1, 1)
end

function mouse()
	local x, y = 474, 217
	if love.mouse.isDown( "l" ) then
		if is_inside(x,    y) then player.run_speed[1] = player.run_speed[1] - (125 * min_dt) end
		if is_inside(x+67, y) then player.run_speed[1] = player.run_speed[1] + (125 * min_dt) end
		if is_inside(x,    y+15) then player.run_speed[2] = player.run_speed[2] - (125 * min_dt) end
		if is_inside(x+67, y+15) then player.run_speed[2] = player.run_speed[2] + (125 * min_dt) end
		if is_inside(x,    y+30) then player.run_speed[3] = player.run_speed[3] - (125 * min_dt) end
		if is_inside(x+67, y+30) then player.run_speed[3] = player.run_speed[3] + (125 * min_dt) end

		if is_inside(x+140,    y)    then player.run_accel[1] = player.run_accel[1] - (125 * min_dt) end
		if is_inside(x+140+67, y)    then player.run_accel[1] = player.run_accel[1] + (125 * min_dt) end
		if is_inside(x+140,    y+15) then player.run_accel[2] = player.run_accel[2] - (125 * min_dt) end
		if is_inside(x+140+67, y+15) then player.run_accel[2] = player.run_accel[2] + (125 * min_dt) end
		if is_inside(x+140,    y+30) then player.run_accel[3] = player.run_accel[3] - (125 * min_dt) end
		if is_inside(x+140+67, y+30) then player.run_accel[3] = player.run_accel[3] + (125 * min_dt) end

		if is_inside(x+140+140,    y)    then player.run_times[1] = player.run_times[1] - (125 * min_dt / 200) end
		if is_inside(x+140+140+67, y)    then player.run_times[1] = player.run_times[1] + (125 * min_dt / 200) end
		if is_inside(x+140+140,    y+15) then player.run_times[2] = player.run_times[2] - (125 * min_dt / 200) end
		if is_inside(x+140+140+67, y+15) then player.run_times[2] = player.run_times[2] + (125 * min_dt / 200) end

		if is_inside(x+140+140+140,    y)    then player.friction = player.friction - (125 * min_dt) end
		if is_inside(x+140+140+140+67, y)    then player.friction = player.friction + (125 * min_dt) end
	end
end

function is_inside(x, y)
	if mx >= x and mx <= x + 25 and my >= y and my <= y+10 then
		return true
	else
		return false
	end
end

function Player:collision_check(dt)
-- collision detection here

	--local tile
	self.y_vel = self.y_vel + (self.gravity * dt)

	local next_x = self.x + (self.x_vel * dt)

	-- X-AXIS --

	if self.x_vel > 0 then
		if	collision(1, next_x + (self.width/2) - self.collision_x_offset, self.y + (self.height / 2) - self.collision_y_offset) or
			collision(1, next_x + (self.width/2) - self.collision_x_offset, self.y) or
			collision(1, next_x + (self.width/2) - self.collision_x_offset, self.y - (self.height / 2) + 1 + self.collision_y_offset) then

			if 	collision(2, next_x + (self.width/2) - self.collision_x_offset - (tile_size/2), self.y + (self.height / 2) - self.collision_y_offset) or
				collision(2, next_x + (self.width/2) - self.collision_x_offset - (2 * (tile_size/2)), self.y + (self.height / 2) - self.collision_y_offset) then

				self.x = next_x
			else
				self.x_vel = 0
				self.x = (math.ceil((next_x + (self.width/2) - self.collision_x_offset) / tile_size) - 1) * tile_size - (self.width / 2) + self.collision_x_offset
			end
		else
			self.x = next_x
			-- check if there is slope in front of you; Give it few additional pixels
			if	collision(2, next_x - (self.width/2) + self.collision_x_offset + 1, self.y + (self.height / 2) + 6) and self.on_floor then
				self.on_slope = true
				self.y_vel = 0
			end
		end
	end

	if self.x_vel < 0 then
		if	collision(1, next_x - (self.width/2) + self.collision_x_offset, self.y + (self.height / 2) - self.collision_y_offset) or
			collision(1, next_x - (self.width/2) + self.collision_x_offset, self.y) or
			collision(1, next_x - (self.width/2) + self.collision_x_offset, self.y - (self.height / 2) + 1 + self.collision_y_offset) then
			-- if player collide with wall check if there is slope behind it
			if 	collision(2, next_x - (self.width/2) + self.collision_x_offset + (tile_size/2), self.y + (self.height / 2) - self.collision_y_offset) or
				collision(2, next_x - (self.width/2) + self.collision_x_offset + (2 * (tile_size/2)), self.y + (self.height / 2) - self.collision_y_offset) then
				
				self.x = next_x
			else
				self.x_vel = 0
				self.x = math.ceil((next_x - (self.width/2) + self.collision_x_offset) / tile_size) * tile_size + (self.width / 2) - self.collision_x_offset
			end
		else
			self.x = next_x
			-- check if there is slope in front of you; Give it few additional pixels
			if	collision(2, next_x + (self.width/2) - self.collision_x_offset - 1, self.y + (self.height / 2) + 6) and self.on_floor then
				self.on_slope = true
				self.y_vel = 0
			end
		end
	end

	 next_y = self.y + (self.y_vel * dt)

	-- Y-AXIS --
	local has_droped = false
	if self.y_vel > 0 then
		--[[ NOTE:
				You can fall on the edge of SOLID block between two slopes.
				In that case you will stand on that block, not on the slope.
				To prevent that we need to check if there's slope few pixels above and few pixels behind.
				If there is - it means player felt through slope and we need to put him on_slope
				to correct his position in later section.
		]]
		-- regular blocks
		if	collision(1, self.x + (self.width/2) - self.collision_x_offset, next_y + (self.height / 2)) or
			collision(1, self.x - (self.width/2) + self.collision_x_offset + 1, next_y + (self.height / 2)) then
			if not self.on_floor then self.drop_speed = self.y_vel; has_droped = true end
			if not ((collision(2, self.x + (self.width/2) - self.collision_x_offset, next_y + (self.height / 2) - (tile_size/4)) and
					collision(2, self.x + (self.width/2) - self.collision_x_offset - (tile_size/4), next_y + (self.height / 2)))
						or
					(collision(2, self.x - (self.width/2) + self.collision_x_offset + 1, next_y + (self.height / 2) - (tile_size/4)) and
					 collision(2, self.x - (self.width/2) + self.collision_x_offset + 1 + (tile_size/4), next_y + (self.height / 2)))) then

				self.y = (math.ceil((next_y + (self.height / 2)) / tile_size) - 1) * tile_size - (self.height / 2)
				self.y_vel = 0
				self.on_floor = true
				self.doublejump = true
			else
				self.on_slope = true
			end
		else
			self.y = next_y
			self.on_floor = false
		end
	end

	if self.y_vel < 0 then
		--if collision(1, self.x - (self.width/2) + self.collision_x_offset + 1,	next_y - (self.height / 2) + self.collision_y_offset) or
		--   collision(1, self.x + (self.width/2) - self.collision_x_offset, 		next_y - (self.height / 2) + self.collision_y_offset) then
		--   self.y_vel = 0
		--else
			self.y = next_y
		--end
	end

	-- SLOPES --

	p1 = {math.round(self.x - (self.width/2) + 3), math.round(next_y + (self.height / 2))}
	p2 = {math.round(self.x + (self.width/2) - 3), math.round(next_y + (self.height / 2))}

	left_slope  = deepcopy(get_tile(p1[1], p1[2]))
	right_slope = deepcopy(get_tile(p2[1], p2[2]))
	if		left_slope['d'] == 'l' then
		slope = get_tile(p1[1], p1[2])
		slope['distance'] = 0
	elseif	right_slope['d'] == 'r' then
		slope = get_tile(p2[1], p2[2])
		slope['distance'] = 0
	else
		-- find the nearest slope for two different points
		-- LEFT side
		-- Grab near tiles
		l_tile1 = deepcopy(get_tile(p1[1] - (tile_size / 2), p1[2]))
		l_tile2 = deepcopy(get_tile(p1[1] + (tile_size / 2), p1[2]))
		l_tile3 = deepcopy(get_tile(p1[1], p1[2] - (tile_size / 2)))
		l_tile4 = deepcopy(get_tile(p1[1] + 4, p1[2] + (tile_size / 2)))

		-- If they are slopes, how far are they?
		if l_tile1['d'] == 'l' then l_tile1['distance'] = math.abs((l_tile1['x'] * tile_size) - p1[1])
		else						l_tile1['distance'] = 1666 end

		if l_tile2['d'] == 'l' then l_tile2['distance'] = math.abs((l_tile2['x'] * tile_size) - tile_size - p1[1])
		else						l_tile2['distance'] = 2666 end

		if l_tile3['d'] == 'l' then l_tile3['distance'] = math.abs((l_tile3['y'] * tile_size) - p1[2])
		else						l_tile3['distance'] = 3666 end

		if l_tile4['d'] == 'l' then l_tile4['distance'] = math.abs((l_tile4['y'] * tile_size) - tile_size - p1[2])
		else						l_tile4['distance'] = 4666 end

		l_nearest_slope = l_tile1
		if l_nearest_slope['distance'] > l_tile2['distance'] then l_nearest_slope = l_tile2 end
		if l_nearest_slope['distance'] > l_tile3['distance'] then l_nearest_slope = l_tile3 end
		if l_nearest_slope['distance'] > l_tile4['distance'] then l_nearest_slope = l_tile4 end
		-- l_nearest_slope is ready!

		 r_tile1 = deepcopy(get_tile(p2[1] - (tile_size / 2), p2[2] + 1))
		 r_tile2 = deepcopy(get_tile(p2[1] + (tile_size / 2), p2[2] - 1))
		 r_tile3 = deepcopy(get_tile(p2[1] - 1, p2[2] - (tile_size / 2)))
		 r_tile4 = deepcopy(get_tile(p2[1] - 4, p2[2] + (tile_size / 2)))
		-- If they are slopes, how far are they?
		if r_tile1['d'] == 'r' then r_tile1['distance'] = math.abs((r_tile1['x'] * tile_size) - p2[1])
		else						r_tile1['distance'] = 6661 end
		
		if r_tile2['d'] == 'r' then r_tile2['distance'] = math.abs((r_tile2['x'] * tile_size) - tile_size - p2[1])
		else						r_tile2['distance'] = 6662 end
		
		if r_tile3['d'] == 'r' then r_tile3['distance'] = math.abs((r_tile3['y'] * tile_size) - p2[2])
		else						r_tile3['distance'] = 6663 end
		
		if r_tile4['d'] == 'r' then r_tile4['distance'] = math.abs((r_tile4['y'] * tile_size) - tile_size - p2[2])
		else						r_tile4['distance'] = 6664 end

		r_nearest_slope = r_tile1
		if r_nearest_slope['distance'] > r_tile2['distance'] then r_nearest_slope = r_tile2 end
		if r_nearest_slope['distance'] > r_tile3['distance'] then r_nearest_slope = r_tile3 end
		if r_nearest_slope['distance'] > r_tile4['distance'] then r_nearest_slope = r_tile4 end
		-- r_nearest_slope is ready!

		if l_nearest_slope['distance'] <= r_nearest_slope['distance'] then
			slope = l_nearest_slope
		else
			slope = r_nearest_slope
		end

		if slope['distance'] > tile_size then slope = {} end
	end

	local i_hit_the_slope_and_i_like_it = false

	if slope['d'] == 'l' and not (self.slope_timer > 0 and self.y_vel < 0) then
	tile = get_tile(p1[1], p1[2])
	t = math.abs((math.floor(p1[1]) - (tile["x"] * tile_size)) / tile_size)

	floor_y = ((slope['l_edge'] - slope['r_edge']) * t) + slope['r_edge']

	-- if player stands on left slope
		if	collision(2, p1[1], p1[2]) and (self.y >= (tile["y"] * tile_size) - (self.height / 2) - math.ceil(floor_y) or self.on_slope) then
			
			debug = "L1"
			self.y = ((tile["y"]) * tile_size) - (self.height / 2) - math.ceil(floor_y)
			i_hit_the_slope_and_i_like_it = true

		elseif collision(1, p1[1], p1[2]) and collision(2, p1[1] + (tile_size / 2), p1[2]) then

			debug = "L2"
			self.y = ((tile["y"] - 1) * tile_size) - (self.height / 2)
			i_hit_the_slope_and_i_like_it = true

		elseif collision(0, p1[1], p1[2]) and collision(2, p1[1], p1[2] + (tile_size / 2)) and self.on_slope then

			debug = "L3"
			slope['l_edge'] = tile_size
			floor_y = ((slope['l_edge'] - slope['r_edge']) * t) + slope['r_edge']
			self.y = ((tile["y"] + 1) * tile_size) - (self.height / 2) - math.ceil(floor_y)
			i_hit_the_slope_and_i_like_it = true

		else
			-- if player walk off the slope stick him to the ground
			if self.on_slope then
				if collision(1, p1[1] + 2, p1[2] + 11) and not (collision(2, p1[1], p1[2])) then
					self.y = (math.ceil(p1[2] / tile_size) * tile_size) - (self.height / 2)
					self.on_floor = true
					self.on_slope = false
				end
			else
				self.on_slope = false
			end
		end

	elseif slope['d'] == 'r' and not (self.slope_timer > 0 and self.y_vel < 0) then

		tile = get_tile(p2[1], p2[2])
		t = -(math.floor(p2[1]) - (tile["x"] * tile_size)) / tile_size

		floor_y = ((slope['l_edge'] - slope['r_edge']) * t) + slope['r_edge']

		if	collision(2, p2[1], p2[2]) and (self.y >= (tile["y"] * tile_size) - (self.height / 2) - math.floor(floor_y) or self.on_slope) and
		slope['d'] == 'r' then
			
			debug = "R1"
			self.y = ((tile["y"]) * tile_size) - (self.height / 2) - math.floor(floor_y)
			i_hit_the_slope_and_i_like_it = true

		elseif collision(1, p2[1], p2[2]) and collision(2, p2[1] - (tile_size / 2), p2[2]) and
		slope['d'] == 'r' then

			debug = "R2"
			self.y = ((tile["y"] - 1) * tile_size) - (self.height / 2)
			i_hit_the_slope_and_i_like_it = true

		elseif collision(0, p2[1], p2[2]) and collision(2, p2[1], p2[2] + (tile_size / 2)) and self.on_slope then

			debug = "R3"
			-- limits high of slope while counting floor_y
			slope['r_edge'] = tile_size
			floor_y = ((slope['l_edge'] - slope['r_edge']) * t) + slope['r_edge']
			self.y = ((tile["y"] + 1) * tile_size) - (self.height / 2) - math.floor(floor_y)
			i_hit_the_slope_and_i_like_it = true

		else
			-- if player walk off the slope stick him to the ground
			if self.on_slope then
				if collision(1, p2[1] - 2, p2[2] + 11) then
					self.y = (math.ceil(p2[2] / tile_size) * tile_size) - (self.height / 2)
					self.on_floor = true
					self.on_slope = false
				end
			else
				self.on_slope = false
			end
		end
	else
		self.on_slope = false
	end
--[[

	If on slope, save current_slope for shizzle and fizzle.

]]

	if self.on_slope then
	-- Left \ slope
		if slope['l_edge'] == 24 and slope['r_edge'] == 1 then
			self.current_slope = {2, 'l'}
	-- Left -- slope
		elseif (slope['l_edge'] == 12 and slope['r_edge'] == 1) or (slope['l_edge'] == 24 and slope['r_edge'] == 12) then
			self.current_slope = {1, 'l'}
	-- Left | slope
		elseif (slope['l_edge'] == 48 and slope['r_edge'] == 1) or (slope['l_edge'] == 24 and slope['r_edge'] == -24) then
			self.current_slope = {3, 'l'}
	-- Right / slope
		elseif slope['r_edge'] == 24 and slope['l_edge'] == 1 then
			self.current_slope = {2, 'r'}
	-- Right -- slope
		elseif (slope['r_edge'] == 12 and slope['l_edge'] == 1) or (slope['r_edge'] == 24 and slope['l_edge'] == 12) then
			self.current_slope = {1, 'r'}
	-- Right | slope
		elseif (slope['r_edge'] == 48 and slope['l_edge'] == 1) or (slope['r_edge'] == 24 and slope['l_edge'] == -24) then
			self.current_slope = {3, 'r'}
		end
	else
		self.current_slope = {}
	end

--[[

	Po opadnięciu na rampę oblicz bonusową prędkość.
	Jeśli prędkość będzie odpowiednio duża - is_sliding = TRUE
	Podczas slajdu nie można poruszać się w przeciwną stronę
	można jedynie biec w tym samym kierunku lub wyjść ze slajdu skokiem.
	Jeśli is_sliding, a dotrzemy do odpowiednio małej prędkości - is_sliding = FALSE

]]
	if not self.on_slope then self.hit_slope = false end
	if i_hit_the_slope_and_i_like_it then
		if not self.hit_slope then
			self.hit_slope = true
			if not has_droped then self.drop_speed = self.y_vel end
			zzz = (1 + (self.y_vel - 500) / 500)
				 bonus_x_vel = 0

		-- Left / slope
			if slope['l_edge'] == 24 and slope['r_edge'] == 1 then

				bonus_x_vel = 75 * zzz
			
		-- Left -- slope
			elseif (slope['l_edge'] == 12 and slope['r_edge'] == 1) or (slope['l_edge'] == 24 and slope['r_edge'] == 12) then

				bonus_x_vel = 75 * zzz * 0.5
			
		-- Left | slope
			elseif (slope['l_edge'] == 48 and slope['r_edge'] == 1) or (slope['l_edge'] == 24 and slope['r_edge'] == -24) then
				
				bonus_x_vel = 75 * zzz * 1.5

		-- Right / slope
			elseif slope['r_edge'] == 24 and slope['l_edge'] == 1 then

				bonus_x_vel = -75 * zzz
			
		-- Right -- slope
			elseif (slope['r_edge'] == 12 and slope['l_edge'] == 1) or (slope['r_edge'] == 24 and slope['l_edge'] == 12) then

				bonus_x_vel = -75 * zzz * 0.5
			
		-- Right | slope
			elseif (slope['r_edge'] == 48 and slope['l_edge'] == 1) or (slope['r_edge'] == 24 and slope['l_edge'] == -24) then
				
				bonus_x_vel = -75 * zzz * 1.5
			end

			if self.is_skiing then
				bonus_x_vel = bonus_x_vel + (bonus_x_vel * 0.35)
			end

			if math.abs(bonus_x_vel) > 90 then
				self.is_sliding = bonus_x_vel
				if self.x_vel > 0 and bonus_x_vel > 0 then
					self.x_vel = self.x_vel + bonus_x_vel
				elseif self.x_vel < 0 and bonus_x_vel < 0 then
					self.x_vel = self.x_vel + bonus_x_vel
				else
					self.x_vel = bonus_x_vel
				end
			else
				self.x_vel = self.x_vel + bonus_x_vel
			end
		end
		self.on_slope = true
		self.on_floor = true
		self.doublejump = true
		self.y_vel = 0
	end

	-- Ceiling slopes D:

--	local p1 = {math.round(self.x - (self.width/2) + 3), math.round(next_y - (self.height / 2))}
--	local p2 = {math.round(self.x + (self.width/2) - 3), math.round(next_y - (self.height / 2))}
--
--	local ceil_slope
--	if collision(3, p1[1], p1[2]) then
--		ceil_slope = get_tile(p1[1], p1[2])
--	else
--		if collision(3, p1[1] + 5, p1[2]) then
--			ceil_slope = get_tile(p1[1] + 5, p1[2])
--		else
--			ceil_slope = nil
--		end
--	end
--
--	if ceil_slope ~= nil then
--		u = math.abs((math.floor(p1[1]) - (ceil_slope['x'] * tile_size)) / tile_size)
--		ceil_y = ((ceil_slope['l_edge'] - ceil_slope['r_edge']) * u) + ceil_slope['r_edge']
--		if	collision(3, p1[1], p1[2]) and self.y - (self.height / 2) <= ((ceil_slope['y'] - 1) * tile_size) + math.ceil(ceil_y) then
--			if self.y_vel < 0 then self.y_vel = 0 end
--			self.y = ((ceil_slope['y']) * (tile_size - 1)) + (self.height / 2) + math.floor(ceil_y)
--		end
--		if	collision(1, p1[1], p1[2]) and collision(3, p1[1] + 5, p1[2]) then
--			if self.y_vel < 0 then self.y_vel = 0 end
--			self.y = ((ceil_slope['y']) * (tile_size - 1)) + (self.height / 2) + math.floor(ceil_y)
--		end
--	end
end